package de.sm.misc;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;

/**
 * Registry implementing basic functionality to get items by name.
 * @author Stefan Marr
 * @param <TId> id type for the registry
 * @param <TValue> item type for the registry
 */
public abstract class Registry<TId, TValue> {
	
	/**
	 * Maps ids to items.
	 */
	protected HashMap<TId, TValue> registry	= new HashMap<TId, TValue>();
	
	/**
	 * Implement creation of items not stored in registry.
	 * @param id of item
	 * @return new item
	 */
	protected abstract TValue getNewItem(TId id);
	
	/**
	 * Get item by name.
	 * @param id of the item
	 * @return item
	 */
	public final TValue get(final TId id) {
		if (!registry.containsKey(id)) {
			TValue item = getNewItem(id);
			registry.put(id, item);
			return item;
		}
		return registry.get(id);
	}
	
	/**
	 * Get all items.
	 * @return all items
	 */
	public final Collection<TValue> getAll() {
		return registry.values();
	}
	
	/**
	 * Get items by name.
	 * 
	 * @param ids of items
	 * @return all named items
	 */
	public final Collection<TValue> get(final TId[] ids) {
		Collection<TValue> items = new ArrayList<TValue>();
		
		for (TId id : ids) {
			items.add(get(id));
		}
		return items;
	}
}
