grammar Odl;

options {
	language = Java;
	output = AST;
}

tokens {
	CLASS 	= 'class';
	TRAIT		= 'trait';
	REFINE	= 'refine';
	BEFORE	= 'before';
	CONST		= 'const';
	RESTRICT= 'restrict';
	ZERO_ARR = '[0]';
}

@header {
  package de.sm.odl;
  import de.sm.odl.elements.*;
  import de.sm.vmadl.elements.ServiceModule;
}

@lexer::header {
  package de.sm.odl;
}

@members {
	private TypeRegistry reg;
	private ServiceModule module;
}

definitions [ TypeRegistry reg, ServiceModule module ]:
	{ this.reg		= $reg;
	  this.module = $module; 
	}
	(classType | traitType | refineType | NEWLINE)* ;
	
classType returns [ClassType t]:
	CLASS 
	{ $t = new ClassType(); 
		$t.setModule(module);
		module.types.add($t); 
	} 
	type[$t]
;

traitType returns [TraitType t]:
	TRAIT
	{ $t = new TraitType();
	  $t.setModule(module);
	  module.types.add($t);
	} 
	type[$t]
;

refineType returns [Type t]:
	REFINE IDENTIFIER { $t = reg.get($IDENTIFIER.text); }
	LBRACE content[$t] RBRACE
;

type [ Type t ]: 
	IDENTIFIER {$t.setName($IDENTIFIER.text); reg.register($t); }
	inheritance[$t] 
	LBRACE content[$t] RBRACE 
;

inheritance [Type t]:
	(COLON baseclass=IDENTIFIER {$t.setBaseclass(reg.get($baseclass.text)); }
	 (',' trait=IDENTIFIER {$t.addTrait((TraitType)reg.get($trait.text));} )*
	)?
; 

content [ Type t ]:
  (NEWLINE | field[$t] | function[$t] )*
;

typeRef returns [ TypeRef ref ]:
	{ $ref = new TypeRef(); }
	(CONST { $ref.constQ = true; })?
	IDENTIFIER { $ref.name = $IDENTIFIER.text; }
	('*' { $ref.incPtr(); } )*
	(RESTRICT { $ref.restrictQ = true; })?
	(CONST { $ref.constQ2 = true; })?
;

fieldname:
	IDENTIFIER ZERO_ARR?;

field [ Type t ]:
	{ Field f = new Field(); }
	typeRef						{ f.type = $typeRef.ref; } 
	name=fieldname		{ f.name = $name.text; }
	(LBRACE BEFORE		{ f.positionPredicate = $BEFORE.text; } 
					pos=fieldname { f.positionField = $pos.text; }
	RBRACE)?
	NEWLINE
	{ $t.addField(f); }
;

function [ Type t ]:
	returnType=typeRef name=IDENTIFIER
	{ Function f = new Function();
		f.name = $name.text;
		f.returnType = $returnType.ref;
		$t.addFunction(f);
	} 
	LPARENTHESIS
		(paramType=typeRef paramName=IDENTIFIER?	{f.addParam(($paramName != null)?$paramName.text:null, $paramType.ref);} 
			(',' moreParam=typeRef moreName=IDENTIFIER? {f.addParam(($moreName != null)?$moreName.text:null, $moreParam.ref);}
			)*
		)? 
	RPARENTHESIS
	NEWLINE
;

POSITION: BEFORE;

IDENTIFIER : (LETTER | '_') ( DIGIT | LETTER | '_' )*;

DIGIT
    :    '0'..'9'
    ;
fragment LETTER
    : 'a'..'z'
    | 'A'..'Z'
    ;

NEWLINE
  : '\r'? '\n'
  //{ channel=HIDDEN; }
;

WS
  : (' '|'\t')+		//|'\n'|'\r' 
  { channel=HIDDEN; }
;

COMMENT
  : '//' (~('\n'|'\r'))* NEWLINE
  { channel=HIDDEN; }
;
  
// multiple-line comments
ML_COMMENT
  : '/*' (options {greedy=false;} : .)* '*/' 
  { channel=HIDDEN; }
;

LPARENTHESIS: '(';
RPARENTHESIS: ')';
COLON  : ':'; 
LBRACE : '{';
RBRACE : '}';