/**
 * 
 */
package de.sm.vmadl;

import java.util.Arrays;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;

import de.sm.odl.Odl;
import de.sm.vmadl.elements.ServiceModule;

/**
 * @author Stefan Marr
 *
 */
public class Main {
	
	/**
	 * Constructur needs Vmadl instance as compiler.
	 * @param vmadl compiler instance
	 * @param odl compiler instance
	 */
	public Main(final Vmadl vmadl) {
		this.vmadl = vmadl;
	}

	//private boolean testFlag;
	private Options options;
	private String[] modules;
	private String outputDir;
	private String sourceDir;
	private Vmadl vmadl;

	//private String getCurrentWorkingDirectory()  {
	//	return System.getProperty("user.dir");
	//}
	
	/**
	 * @param args
	 */
	public static void main(final String[] args) {
//		try {
			Vmadl vmadl = new Vmadl(new ServiceRegistry(),
					new CombinationRegistry(), Odl.getInstance());
			Main app = new Main(vmadl);
			app.run(args);
//		} catch (Exception e) {
//			e.printStackTrace();
//			System.exit(-1);
//		}
	}
	
	public final void run(final String[] args) {
		createOptions();
		
		// create the command line parser
		CommandLineParser parser = new PosixParser();
		CommandLine line;
		try {
			line = parser.parse(options, args);
			
			//testFlag = line.hasOption("test");
			//String outputPath = line.getOptionValue('o');
			//String outputFilename = line.getOptionValue('f');
			
			String[] arguments = line.getArgs();
			
			if (arguments.length < 3 || line.hasOption('?')) {
				printUsage();
				return;
			}
			
			modules = Arrays.copyOfRange(arguments, 0, arguments.length - 2);
			sourceDir = arguments[arguments.length - 2];
			outputDir = arguments[arguments.length - 1];
			
			
			loadVmadl();
			generateImplementation();
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			printUsage();
		}
	}
	
	private void loadVmadl() {
		vmadl.setSourceDir(sourceDir);
		vmadl.loadFiles();
		vmadl.setUsedModules(modules);
	}
	
	/**
	 * Generate the implementation from the already loaded Vmadl definition.
	 */
	private void generateImplementation() {
		try {
			vmadl.checkConsistency();
			vmadl.compile();
			vmadl.generateImplementation(outputDir);
		} catch (CompilerException e) {
			ServiceModule m = e.getModule();
			System.err.println(m.file.getAbsolutePath() + ": error "
					+ e.getMessage());
			System.exit(-1);
		}
	}

	/**
	 * Initialize the command-line parameter options.
	 */
	private void createOptions() {
		options = new Options();
//		options.addOption("test", false, "if the test option is set,"
//				+ " output to test generator is produced");
//		options.addOption("f", "file", true, "output will be generated"
//				+ " into files with given prefix if this option isn't used,"
//				+ " output will be splited up into separate files for"
//				+ " each service.");
//		options.addOption("o", "output-path", true, 
//				"output will be generated into given folder");
		options.addOption("?", "help", false, "prints this help message");
	}
	
	/**
	 * Output command-line help to the user.
	 */
	private void printUsage() {
		HelpFormatter formatter = new HelpFormatter();
		formatter.printHelp("vmadl <service-name1> [... <service-nameN>]"
				+ " <source-folder> <output-folder>",
				"vmadl will compile the given VM architecture description "
				+ "into c-header and AspectC++-aspect files."
				+ " With the current ODL support types {} sections in VMADL "
				+ "are compiled to CSOM-Style object "
				+ " support files. VTables and structures are generated.\n\n "
				+ "<source-folder> is used to be able to generate properer "
				+ "VTable initializers and to find and read the vmadl files."
				+ " The names of the VMADL files without there extension will"
				+ " be considerered as source folders to be included."
				, options, "", true);
		
	}
}


