
#include <stdio.h>
#include <unistd.h>

#include "Scheduler.h"

#include <vm/Universe.h>

#include <green-threads/Scheduler.h>

#include <interpreter/bytecodes.h>
#include <interpreter/Interpreter.h>

#include <vmobjects/VMMethod.h>
#include <vmobjects/VMArray.h>
#include <vmobjects/VMClass.h>
#include <vmobjects/VMFrame.h>
#include <vmobjects/VMInvokable.h>
#include <vmobjects/VMInteger.h>
#include <vmobjects/VMBigInteger.h>
#include <vmobjects/VMSymbol.h>

#include <memory/memory.h>

extern "C" {

void  _Scheduler_runThread_(pVMObject object, pVMFrame frame) {
	pVMObject arg = SEND(frame, pop);
	pVMObject self = SEND(frame, pop);

	Scheduler_run_thread(arg, frame);
}

void  _Scheduler_registerThreadFrame_(pVMObject object, pVMFrame frame) {
	pVMObject arg = SEND(frame, pop);
	pVMObject self = SEND(frame, pop);
	
	Scheduler_register_thread_frame(arg);
}

void  _Scheduler_unregisterThreadFrame_(pVMObject object, pVMFrame frame) {
	pVMObject arg = SEND(frame, pop);
	pVMObject self = SEND(frame, pop);
	
	Scheduler_unregister_thread_frame(arg);
}

void _Scheduler_enterCriticalSection(pVMObject object, pVMFrame frame) {
	Interpreter_enter_uninterruptable_section();	
}

void _Scheduler_leaveCriticalSection(pVMObject object, pVMFrame frame) {
	Interpreter_leave_uninterruptable_section();
}

void _Scheduler_runSchedulerThread() {
	Scheduler_run_scheduler_thread();
}

void _Scheduler_sleep_(pVMObject object, pVMFrame frame) {
	pVMObject arg = SEND(frame, pop);
	pVMObject self = SEND(frame, pop);
	if (IS_A(arg, VMInteger) || IS_A(arg, VMBigInteger)) {
		long time = (long)SEND((pVMInteger)arg, get_embedded_integer);
		Scheduler_sleep(time);	
	}
}

}
