#ifndef CORE_SYSTEM_H_
#define CORE_SYSTEM_H_

/*
 *
Copyright (c) 2007 Michael Haupt, Tobias Pape
Software Architecture Group, Hasso Plattner Institute, Potsdam, Germany
http://www.hpi.uni-potsdam.de/swa/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
  */

#include <vmobjects/OOObject.h>
#include <time.h>

extern "C" {

void _System_global_(pVMObject object, pVMFrame frame);
void _System_global_put_(pVMObject object, pVMFrame frame);
void _System_load_(pVMObject object, pVMFrame frame);
void _System_exit_(pVMObject object, pVMFrame frame);
void _System_printString_(pVMObject object, pVMFrame frame);
void _System_printNewline(pVMObject object, pVMFrame frame);
void _System_time(pVMObject object, pVMFrame frame);


/**
 * \brief Hibernates the currently running VM image.
 * 
 * Stores the currently running VM image in a file, so
 * enabling its later reactivation.
 * 
 * The function has one "string argument", the name of
 * the image, from which its filename is derived. This
 * name is also needed during loading.
 * 
 * \note This method is not expected to return. (Though
 * on Smalltalk-level it does - when the image is loaded.)
 */ 
void _System_hibernate_withEncryption_usingKey_(pVMObject object, pVMFrame frame);

/**
 * \brief Inspects an image with the given name.
 * 
 * Returns a Smalltalk object (VMImage on VM side),
 * providing access to meta data of the image.
 */
void _System_inspectImage_(pVMObject object, pVMFrame frame);

/**
 * \brief Restores the image with the given name.
 * 
 * Restores an image from file. The image name (and thus also
 * the file name) is given by the first "string argument".
 * 
 * On Smalltalk level this method is not expected to return.
 * But, on a C level it does, since "only" the Smalltalk object
 * world and interpreter stack has been switched.
 */
void _System_loadImage_usingKey_(pVMObject object, pVMFrame frame);

/**
 * \brief Relocates the heap to a new location.
 * 
 * Moves the heap to a new location to test the relocation
 * functionality.
 */
void _System_rh(pVMObject object, pVMFrame frame);

void __System_init(void);
extern struct timeval _System_start_time;

}

#endif CORE_SYSTEM_H_



