/*
 *
Copyright (c) 2007 Michael Haupt, Tobias Pape
Software Architecture Group, Hasso Plattner Institute, Potsdam, Germany
http://www.hpi.uni-potsdam.de/swa/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
 */

#include <vm/Universe.h>


#include "memory.h"

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


void* internal_allocate(size_t size) {
    if(size == 0)
        return NULL;
    void* result = malloc(size);
    if(!result) {
        debug_error("Failed to allocate %d bytes. Panic.\n", (int)size);
        Universe_exit(-1);
    }
    memset(result, 0, size);
    return result;
}


void internal_free(void* ptr) {
    free(ptr);
}


char* internal_allocate_string(const char* restrict str) {
    return strdup(str);
}


void* memory_allocate(size_t size) {
    if(size == 0)
        return NULL;
    void* result = malloc(size);
    if(!result) {
        debug_error("Failed to allocate %d bytes. Panic.\n", (int)size);
        Universe_exit(-1);
    }
    memset(result, 0, size);
    return result;
}


void* memory_allocate_object(size_t size) {
    void* o = memory_allocate(size);
    if(o)
        ((pOOObject)o)->object_size = size;
    return o;
}


void memory_free(void* ptr) {
    internal_free(ptr);
}
