/* Native multi-threading for CSOM - Signal primitive class
 * 
 * Copyright (C) 2007 by Martin Beck and Jan Klimke
 * 
 * created 2007-06-11
 */
 
#include "VMSignal.h"

#include <memory/memory.h>

#include <vm/Universe.h>

#include <stdbool.h>
#include <stdio.h>

pSignalId _VMSignal_get_embedded_signal_id(pVMSignal);
pMutexId  _VMSignal_get_embedded_mutex_id(pVMSignal);
void      _VMSignal_wait(pVMSignal);
void      _VMSignal_signal(pVMSignal);
bool      _VMSignal_signal_all(pVMSignal);

void _VMSignal_init(void* _self, ...);
void _VMSignal_free(void *self);

pVMSignal VMSignal_new() {
	pVMSignal result = (pVMSignal) memory_allocate(sizeof(VMSignal));
    if(result) {
        result->_vtable = VMSignal_vtable();
        INIT(result);
    }
    return result;
}

void _VMSignal_init(void* _self, ...) {
	pVMSignal self = (pVMSignal)_self;
	
    SUPER(VMObject, self, init, 0);
    
	// create a new pthread signal and embed it in the CSOM signal
	int err = pthread_cond_init(&(self->embedded_signal_id), NULL);
	if(err != 0){
        fprintf(stderr, "Could not initialise signal.\n");
	}
	
	err = pthread_mutex_init(&(self->embedded_mutex_id), NULL);
	
	SEND(self, set_class, signal_class);
}

void _VMSignal_free(void *self) {
    // nothing to declare
}

pSignalId _VMSignal_get_embedded_signal_id(pVMSignal signal_object) {
	return &signal_object->embedded_signal_id;
}

void _VMSignal_set_embedded_signal_id(pVMSignal signal_object, SignalId value) {
	signal_object->embedded_signal_id = value;
}

pMutexId _VMSignal_get_embedded_mutex_id(pVMSignal signal_object) {
	return &signal_object->embedded_mutex_id;
}

void _VMSignal_wait(pVMSignal signal_object) {
	pSignalId psid = SEND(signal_object, get_embedded_signal_id);
	pMutexId pmid = SEND(signal_object, get_embedded_mutex_id);
	pthread_mutex_lock(pmid);
	pthread_cond_wait(psid, pmid);
	pthread_mutex_unlock(pmid);
}


void _VMSignal_signal(pVMSignal signal_object) {
	pSignalId psid = SEND(signal_object, get_embedded_signal_id);
	pMutexId pmid = SEND(signal_object, get_embedded_mutex_id);
	pthread_mutex_lock(pmid);
	pthread_cond_signal(psid);
	pthread_mutex_unlock(pmid);
}

bool _VMSignal_signal_all(pVMSignal self) {
	return pthread_cond_broadcast(&self->embedded_signal_id);
}

static VTABLE(VMSignal) _VMSignal_vtable;
bool   VMSignal_vtable_inited = false;

VTABLE(VMSignal) *VMSignal_vtable(void) {
    if(!VMSignal_vtable_inited) {
        *((VTABLE(VMObject)*)&_VMSignal_vtable) = *VMObject_vtable();
        _VMSignal_vtable.init = METHOD(VMSignal, init);
        _VMSignal_vtable.free = METHOD(VMSignal, free);
        
        _VMSignal_vtable.get_embedded_signal_id =
            METHOD(VMSignal, get_embedded_signal_id);
        _VMSignal_vtable.get_embedded_mutex_id =
            METHOD(VMSignal, get_embedded_mutex_id);
        _VMSignal_vtable.wait       = METHOD(VMSignal, wait);
        _VMSignal_vtable.signal     = METHOD(VMSignal, signal);
        _VMSignal_vtable.signal_all = METHOD(VMSignal, signal_all);
        
        VMSignal_vtable_inited = true;
    }
    
    return &_VMSignal_vtable;
}
