/* 
 * Mutex.c
 *  
 * Native multi-threading synchronization for CSOM
 * 
 * by Martin Beck and Jan Klimke
 * 
 * created 2007-05-10
 */

#include "Mutex.h"

#include <vmobjects/VMObject.h>
#include <vmobjects/VMFrame.h>
#include <vmobjects/VMClass.h>

#include <native/VMMutex.h>

#include <vm/Universe.h>

extern "C" {

void _Mutex_lock(pVMObject object, pVMFrame frame){
	pVMMutex mutex = (pVMMutex)SEND(frame, pop);
	pMutexId pmid = SEND(mutex, get_embedded_mutex_id);
    SEND(mutex, lock);
    SEND(frame, push, (pVMObject)mutex);
}


void _Mutex_unlock(pVMObject object, pVMFrame frame){
	pVMMutex mutex = (pVMMutex)SEND(frame, pop);
	pMutexId pmid = SEND(mutex, get_embedded_mutex_id);
    SEND(mutex, unlock);
	SEND(frame, push, (pVMObject)mutex);	
}

void _Mutex_isLocked(pVMObject object, pVMFrame frame){
	pVMMutex mutex = (pVMMutex)SEND(frame, pop);
	pMutexId pmid = SEND(mutex, get_embedded_mutex_id);
	//try to lock the mutex
	if (SEND(mutex, is_locked)) {
		SEND(frame, push, (pVMObject)true_object);
	} else {
		SEND(frame, push, (pVMObject)false_object);
	}
}

void Mutex_new(pVMObject object, pVMFrame frame){
	pVMClass self = (pVMClass) SEND(frame, pop);
	pVMMutex mutex = (pVMMutex)VMMutex_new();
    SEND(frame, push, (pVMObject)mutex);
}

}
