#ifndef VMOBJECT_H_
#define VMOBJECT_H_

/*
 *
Copyright (c) 2007 Michael Haupt, Tobias Pape
Software Architecture Group, Hasso Plattner Institute, Potsdam, Germany
http://www.hpi.uni-potsdam.de/swa/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
  */

#include <vmobjects/OOObject.h>

#include <misc/String.h>

#include <stdbool.h>

#pragma mark VM Allocation Helper


#pragma mark VTable definition

VTABLE(VMObject) {
#define VMOBJECT_VTABLE_FORMAT \
    OOOBJECT_VTABLE_FORMAT; \
    pVMClass  (*get_class)(void*); \
    void      (*set_class)(void*, pVMClass); \
    pVMSymbol (*get_field_name)(void*, int); \
    int       (*get_field_index)(void*, pVMSymbol); \
    int       (*get_number_of_fields)(void*); \
    void      (*set_number_of_fields)(void*, int); \
    int       (*get_default_number_of_fields)(void*); \
    void      (*send)(void*, pString, pVMObject*, int); \
    pVMObject (*get_field)(void*, int); \
    void      (*set_field)(void*, int, pVMObject)
    
    VMOBJECT_VTABLE_FORMAT;
};

#pragma mark class definition

#define VMOBJECT_FORMAT \
    OOOBJECT_FORMAT; \
    size_t     num_of_fields; \
    uint32_t   ref_count; \
    pVMObject  fields[0]; \
    pVMClass   clazz


/*
 * This is the number of fields visible to Smalltalk objects from the Smalltalk
 * side of the world.
 */
#ifdef __puma
#define VMOBJ_STUB ((sizeof(VMObject)/sizeof(pVMObject)) - 1)
#define NUMBER_OF_OBJECT_FIELDS(type) ( (sizeof(type)/sizeof(pVMObject)) - VMOBJ_STUB ) 
#else
#define NUMBER_OF_OBJECT_FIELDS(type) ((sizeof(type) - offsetof(type, fields)) / sizeof(pVMObject))
#endif

struct _VMObject {
    VTABLE(VMObject)* _vtable;
    VMOBJECT_FORMAT;
};

#pragma mark class methods

pVMObject VMObject_new(void);
pVMObject VMObject_new_num_fields(size_t);
void      VMObject_assert(bool);
pVMObject VMObject_new_instance(pVMClass instance_class);

#pragma mark vtable initialization

VTABLE(VMObject)* VMObject_vtable(void);

#endif VMOBJECT_H_

