#ifndef VMINTEGERUTILS_H_
#define VMINTEGERUTILS_H_

/*
 *
Copyright (c) 2007 Michael Haupt, Tobias Pape
Software Architecture Group, Hasso Plattner Institute, Potsdam, Germany
http://www.hpi.uni-potsdam.de/swa/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
  */

#if DEBUG
#include <stdio.h>
#endif

#define USE_TAGGED_POINTERS 1
#define ENABLE_TAGGED_INTEGERS 1

/**
 * max value for tagged integers
 * 01111111 11111111 11111111 1111111X  
 */
#define VMTAGGEDINTEGER_MAX  1073741823

/**
 * min value for tagged integers
 * 10000000 00000000 00000000 0000000X  
 */
#define VMTAGGEDINTEGER_MIN -1073741824


/**
 * check whether a given "object reference" is a tagged integer value 
 */
#if ENABLE_TAGGED_INTEGERS
	#define VMINTEGER_IS_TAGGED(V) ( (!((int32_t)V & 1)) )
#else
	#define VMINTEGER_IS_TAGGED(V) ( (false) )
#endif


/**
 * set an tagged integer
 */
#define VMINTEGER_SET(O,V) \
    ({ \
	 if (VMINTEGER_IS_TAGGED(O)) { O = (typeof(O))VMINTEGER_TAG(V); } \
	 	else { SEND(O,set_embedded_integer,V); } \
    })


/**
 * helper to convert an integer value into a tagged integer value
 */
#if DEBUG
	#define VMINTEGER_TAG(value) \
	({ \
		if(((int32_t)(value)) < VMTAGGEDINTEGER_MIN || ((int32_t)(value)) > VMTAGGEDINTEGER_MAX) { \
			printf("Failed to tag integer %d. Out of range.\n", value);  \
			Universe_exit(ERR_PANIC);									 \
	    } \
		_VMINTEGER_TAG(value); \
	})
#else
	#define VMINTEGER_TAG(value) _VMINTEGER_TAG(value)
#endif
	
#define _VMINTEGER_TAG(value) ((void*)(value << 1))


/**
 * helper to convert a tagged integer value into a untagged integer value
 */
#if DEBUG
	#define VMINTEGER_UNTAG(tagged) \
	({ \
		if(!VMINTEGER_IS_TAGGED(tagged)) { \
			printf("Given value is not a tagged int %d.\n", (int32_t)tagged); \
			Universe_exit(ERR_PANIC);							 \
	    } \
		_VMINTEGER_UNTAG(tagged); \
	})
#else
	#define VMINTEGER_UNTAG(value) _VMINTEGER_UNTAG(value)
#endif

#define _VMINTEGER_UNTAG(tagged) ((int32_t)tagged >> 1)



#endif
