#!/usr/bin/php
<?php
error_reporting(E_ALL);

$successLog = fopen('succ.log', 'a');
$benchmarks = array('IntegerLoop', 'Fibonacci', 'Dispatch', 'Bounce', 'Loop', 'Permute', 'Queens', 'List', 'Recurse',
        'Storage', 'Sieve', 'BubbleSort', 'QuickSort', 'Sum', 'Towers', 'TreeSort');

chdir("odl");

exec("./configure 2>/dev/null", $output);
$features = array();
$banned = array();

foreach($output as $out) {
  $o = split("\t", trim($out));
  if (count($o) > 1) {
    $features[] = trim($o[0]);
  } else {
    $o = split(", ", trim($out));
    if (count($o) > 1) {
      $banned[] = $o;
    }
  }
}

// each bit represents the presents of a specific feature
$combinationCount = pow(2, count($features)); 

$combinations = array();
for ($i = 0; $i < $combinationCount; $i++) {
  $combination = array();
  for ($feature = 0; $feature < count($features); $feature++) {
    if ($i & pow(2, $feature)) {
      $combination[] = $features[$feature];
    }
  }
  $combinations[] = $combination;
}

foreach ($combinations as $combination) {
  if (ommit($combination, $banned)) {
    echo '['.date(DATE_ATOM).'] Ommited Combination: '.join(', ', $combination)."\n";
    continue;
  }

  echo '['.date(DATE_ATOM).'] ';
  echo "Build with following features: ".join(', ', $combination)."\n";
  //string exec  ( string $command  [, array &$output  [, int &$return_var  ]] )
  exec('make clobber');
  if (!empty($combination)) {
    exec('./configure '.join(' ', $combination));
  }
  $idString = join('-', $combination);
  $logId = '../log/'.(empty($idString)?'plain':$idString).'.';

  exec('make prepare > '.$logId.'prepare.log 2>&1');
  exec('make > '.$logId.'make.log 2>&1', $out, $returnValue);
  if ($returnValue == 0) {
    echo '['.date(DATE_ATOM).'] '; echo "Build Succeeded\n";

    fwrite($successLog, '['.date(DATE_ATOM).'] make with '.join(' ', $combination)."\n");
    fflush($successLog);

    exec('make test > '.$logId.'test.log 2>&1', $out, $returnValue);
    if ($returnValue == 0) {
      echo '['.date(DATE_ATOM).'] '; echo "Test Succeeded\n";
      fwrite($successLog, '['.date(DATE_ATOM).'] test with '.join(' ', $combination)."\n");
      fflush($successLog);
    } else {
      echo '['.date(DATE_ATOM).'] ';
      echo "Tests FAILED\n";
    }

    exec('make bench > '.$logId.'prepare.log 2>&1', $out, $returnValue);
    if ($returnValue == 0) {
      echo '['.date(DATE_ATOM).'] '; echo "Benchmarks succeeded\n";
      fwrite($successLog, '['.date(DATE_ATOM).'] bench with '.join(' ', $combination)."\n");
      fflush($successLog);
    } else {
      echo '['.date(DATE_ATOM).'] ';
      echo "Benchmarks FAILED\n";
    }

    $oldPath = getcwd();
    chdir('bin');
    echo "Run Eval Benchmarks\n";
    exec("./CSOM -cp Smalltalk Examples/Benchmarks/All.som > ../$logId.bench.log");
    foreach($benchmarks as $bench) {
        echo "$bench\n";
        for ($i = 0; $i < 10; $i++) {
          echo "$i";
          exec("./CSOM -cp Smalltalk Examples/Benchmarks/$bench.som >> ../$logId.$bench.log");
        }
        echo "\n";
    }

    chdir($oldPath);

  } else {
    echo '['.date(DATE_ATOM).'] ';
    echo "Build FAILED\n";
  }
}

// END of Script code, now helper functions will follow
function ommit($comb, $bannedCombs) {
  foreach ($bannedCombs as $bannedComb) {
    $all = true;
    foreach ($bannedComb as $banned) {
      if (false === in_array($banned, $comb)) {
        $all = false;
        break;
      }
    }
    if ($all) return true;
  }
  return false;
}

fclose($successLog);
