#ifndef VMTHREAD_H_
#define VMTHREAD_H_

/* Native multi-threading for CSOM - Thread primitive class
 * 
 * Copyright (C) 2007 by Martin Beck and Jan Klimke
 * 
 * created 2007-05-09
 */

#include <native-threads/VMSignal.h>

#include <vmobjects/VMObject.h>

#include <pthread.h>

#include "native.h"

typedef pthread_t ThreadId;
typedef pthread_key_t ThreadSafeGlobal;

#pragma mark VTable definition

VTABLE(VMThread) {
#define VMTHREAD_VTABLE_FORMAT \
    VMOBJECT_VTABLE_FORMAT; \
    pVMSignal (*get_resume_signal)(void*); \
    void      (*set_resume_signal)(void*, pVMSignal); \
    bool	  (*should_stop)(void*); \
    void      (*set_should_stop)(void*, bool); \
    pVMBlock  (*get_block_to_run)(void*); \
    void	  (*set_block_to_run)(void*, pVMBlock); \
    pVMString (*get_name)(void*); \
    void	  (*set_name)(void*, pVMString); \
	pVMObject (*get_argument)(void*); \
    void	  (*set_argument)(void*, pVMObject); \
    ThreadId  (*get_embedded_thread_id)(pVMThread); \
    void      (*set_embedded_thread_id)(pVMThread, ThreadId); \
    void      (*join)(pVMThread, int*)
    
    VMTHREAD_VTABLE_FORMAT;
};

#pragma mark class definition

struct _VMThread {
    VTABLE(VMThread) *_vtable;
    
#define VMTHREAD_FORMAT \
    VMOBJECT_FORMAT; \
    pVMSignal resume_signal; \
    pVMObject should_stop; \
    pVMBlock  block_to_run; \
    pVMString name; \
    pVMObject argument; \
    ThreadId embedded_thread_id
    
    VMTHREAD_FORMAT;
};


#pragma mark class methods

pVMThread VMThread_new(void);
void      VMThread_yield(void);
void      VMThread_create_safe_global(ThreadSafeGlobal*);
void      VMThread_delete_safe_global(ThreadSafeGlobal);
void*     VMThread_get_safe_global(ThreadSafeGlobal);
void      VMThread_set_safe_global(ThreadSafeGlobal, void*);

#pragma mark vtable initialization

VTABLE(VMThread)* VMThread_vtable(void);

#endif /*VMTHREAD_H_*/
