#ifndef TAGGED_INT_ONE_H_
#define TAGGED_INT_ONE_H_

#include <stdint.h>

/**
 * always returns an instance ptr, whether it was a tagged int or not.
 */
#define INSTANCE_POINTER_ACCESS(O) \
    ({ (VMINTEGER_IS_TAGGED(O) ? (void*)VMInteger_Global_Box() : O); })

#define SEND(O,M,...) \
    ({  typeof(O) _Org = (typeof(O))(O); \
		typeof(_Org) _O = (typeof(_Org))(INSTANCE_POINTER_ACCESS(_Org)); \
    (_O->_vtable->M(_Org, ##__VA_ARGS__)); \
    })

#ifdef EXPERIMENTAL
#define TSEND(TRAIT,O,M,...) \
    ({ typeof(O) _O = (INSTANCE_POINTER_ACCESS(O)); \
    (((VTABLE(TRAIT)*)(_O->_vtable))->M((TRAIT *)_O , ##__VA_ARGS__)); \
    })
#else
#define TSEND(TRAIT,O,M,...) \
    ({ typeof(O) _O = (typeof(O))(INSTANCE_POINTER_ACCESS(O)); \
    (((VTABLE(TRAIT)*)(_O->_vtable->_ttable))->M((TRAIT*)_O , ##__VA_ARGS__)); \
    })
#endif EXPERIMENTAL

#define IS_A(object,class) \
    (((class *)INSTANCE_POINTER_ACCESS(object))->_vtable == class##_vtable())


#ifdef EXPERIMENTAL
#define SUPPORTS(O,TRAIT) ({ \
    VTABLE(OOObject)* _vt = (VTABLE(OOObject)*)(INSTANCE_POINTER_ACCESS(O)->_vtable); \
    bool found = false; \
    while(_vt->_ttable) { \
        if(_vt->_ttable == TRAIT##_vtable()) \
            found = true; \
             _vt=((VTABLE(OOObject)*)_vt)->_ttable; \
         } \
    found; \
})
#else
#define SUPPORTS(O,TRAIT) \
    ((VTABLE(TRAIT)*)((VTABLE(OOObject)*)((( \
        (pOOObject)INSTANCE_POINTER_ACCESS(O)))->_vtable)->_ttable) == \
            TRAIT##_vtable())
#endif EXPERIMENTAL

#include <stdio.h>
#include <vmobjects/objectformats.h>

/**
 * return a pointer to global integer box, used for all tagged integers
 */
pVMInteger VMInteger_Global_Box(void);

/**
 * max value for tagged integers
 * 01111111 11111111 11111111 1111111X  
 */
const int32_t VMTAGGEDINTEGER_MAX = 1073741823;

/**
 * min value for tagged integers
 * 10000000 00000000 00000000 0000000X  
 */
const int32_t VMTAGGEDINTEGER_MIN = -1073741824;

/**
 * the bit to use for integer tagging
 */
#define VMINTEGER_TAG_BIT 1

/**
 * check whether a given "object reference" is a tagged integer value 
 */
#define VMINTEGER_IS_TAGGED(V) ( ((int32_t)V & VMINTEGER_TAG_BIT) )

/**
 * set an tagged integer
 */
#define VMINTEGER_SET(O,V) \
    ({ \
	 if (VMINTEGER_IS_TAGGED(O)) { O = (typeof(O))VMINTEGER_TAG(V); } \
     else { SEND((pVMInteger)O, set_embedded_integer,V); } \
    })


/**
 * helper to convert an integer value into a tagged integer value
 */
#if DEBUG
	#define VMINTEGER_TAG(v) \
	({ intptr_t value = (intptr_t)v; \
		if (value < VMTAGGEDINTEGER_MIN || value > VMTAGGEDINTEGER_MAX) { \
			printf("Failed to tag integer %p. Out of range. In file %s on line %d Min: %d Max: %d, %p, %p\n", value, __FILE__, __LINE__, VMTAGGEDINTEGER_MIN, VMTAGGEDINTEGER_MAX, value < VMTAGGEDINTEGER_MIN, value > VMTAGGEDINTEGER_MAX);  \
			Universe_exit(ERR_PANIC);									 \
	    } \
		_VMINTEGER_TAG(value); \
	})
#else
	#define VMINTEGER_TAG(value) _VMINTEGER_TAG(value)
#endif
	
#define _VMINTEGER_TAG(value) ((void*)((value << 1) | VMINTEGER_TAG_BIT))


/**
 * helper to convert a tagged integer value into a untagged integer value
 */
#if DEBUG
	#define VMINTEGER_UNTAG(tagged) \
	({ \
		if(!VMINTEGER_IS_TAGGED(tagged)) { \
			printf("Given value is not a tagged int %d.\n", (int32_t)tagged); \
			Universe_exit(ERR_PANIC);							 \
	    } \
		_VMINTEGER_UNTAG(tagged); \
	})
#else
	#define VMINTEGER_UNTAG(tagged) _VMINTEGER_UNTAG(tagged)
#endif

#define _VMINTEGER_UNTAG(tagged) ((int32_t)tagged >> 1)

// TODO: generated this funtion definitions and get ridd of them entierly in hand written code
// overwritten parent methods
// forward definitions are used in VMTable construction, this code is still not modularized, look at VMInteger.c
void _VMInteger_free(void* _self);
int32_t _VMInteger_object_size(void* _self);
pVMClass _VMInteger_get_class(void* _self);
void _VMInteger_set_class(void* _self, pVMClass value);
pVMObject _VMInteger_get_field(void* _self, int index);
void _VMInteger_set_field(void* _self, int index, pVMObject value);
int _VMInteger_get_number_of_fields(void* _self);


#endif TAGGED_INT_ONE_H_
