/*
 *
Copyright (c) 2007 Michael Haupt, Tobias Pape
Software Architecture Group, Hasso Plattner Institute, Potsdam, Germany
http://www.hpi.uni-potsdam.de/swa/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
  */
 
#include "VMString.h"

#include <module.Memory>
#include <module.VM>
#include <module.VMObjects>

//
//  Class Methods (Starting with VMString_) 
//
//

/**
 * Create a new VMString
 */
pVMString VMString_new(void) {
    return VMString_new_with(NULL);
}

pVMString VMString_new_with_chars(const char* restrict chars) {
    return VMString_new_with(String_new(chars));
}

pVMString VMString_new_with(pString string) {
    pVMString result = (pVMString)memory_allocate_object(sizeof(VMString));
    if(result) {
        result->_vtable = VMString_vtable();
        INIT1(result, string);
    }
    return result;
}


/**
 * Initialize a VMString
 */
void _VMString_init(void* _self, void* param1, void* param2) {
    pVMString self = (pVMString)_self;
    size_t size = NUMBER_OF_OBJECT_FIELDS(VMObject);
    SUPER(VMObject, self, init, &size, NULL);
    
    pString embed = String_new_from((pString)param1);
    
    if (embed) {
        SEND(self, set_embedded_string, embed);
    } else {
        self->embedded_string = NULL;
    }
    
    SEND(self, set_class, string_class);
}

void _VMString_free(void* _self) {
    pVMString self = (pVMString)_self;
    if(self->embedded_string)
        SEND(self->embedded_string, free);
    SUPER(VMObject, self, free);
}


//
//  Instance Methods (Starting with _VMString_) 
//
//
pString _VMString_get_embedded_string(void* _self) {
    pVMString self = (pVMString)_self;
    return self->embedded_string;
}


void _VMString_set_embedded_string(void* _self, pString embedded) {
    pVMString self = (pVMString)_self;
    self->embedded_string = embedded;
    self->hash = self->embedded_string->hash;
}


//
// The VTABLE-function
//
//
static VTABLE(VMString) _VMString_vtable;
bool VMString_vtable_inited = false;

VTABLE(VMString)* VMString_vtable(void) {
    if(! VMString_vtable_inited) {
        *((VTABLE(VMObject)*)&_VMString_vtable) = *VMObject_vtable();
        _VMString_vtable.init = METHOD(VMString, init);        
        _VMString_vtable.free = METHOD(VMString, free);
        _VMString_vtable.get_embedded_string =
            METHOD(VMString, get_embedded_string);
        _VMString_vtable.set_embedded_string =
            METHOD(VMString, set_embedded_string);
        
        VMString_vtable_inited = true;
    }
    return &_VMString_vtable;
}
