/*
 *
Copyright (c) 2007 Michael Haupt, Tobias Pape
Software Architecture Group, Hasso Plattner Institute, Potsdam, Germany
http://www.hpi.uni-potsdam.de/swa/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
  */

#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <limits.h>

#include <module.VMObjects>
#include <module.Image>
#include <module.VM>

#include <vmobjects/VMInteger.h>


#include "Image.h"

extern "C" {

void __Image_init(void) {
}

//void  __Image_getCurrentVersion(pVMObject object, pVMFrame frame) {
//    pVMImage img = (pVMImage)SEND(frame, pop);
//    SEND(frame, push, (pVMObject)VMInteger_new_with(IMAGE_MAJOR_VERSION));   
//}

void  _Image_getFilename(pVMObject object, pVMFrame frame) {
    pVMImage img = (pVMImage)SEND(frame, pop);
    pVMString file_name = SEND(img, get_file_name);
    SEND(frame, push, (pVMObject)file_name);   
}

void  _Image_getSize(pVMObject object, pVMFrame frame) {
    pVMImage img = (pVMImage)SEND(frame, pop);
    pVMInteger size = (pVMInteger)SEND(img, get_size);
    SEND(frame, push, (pVMObject)size);   
}

void  _Image_getTimestamp(pVMObject object, pVMFrame frame) {
    pVMImage img = (pVMImage)SEND(frame, pop);
    time_t ts = (time_t)SEND(img, get_timestamp);
    SEND(frame, push, (pVMObject)VMInteger_new_with(ts));   
}

void  _Image_getVersion(pVMObject object, pVMFrame frame) {
    pVMImage img = (pVMImage)SEND(frame, pop);
    int32_t version = (time_t)SEND(img, get_version);
    SEND(frame, push, (pVMObject)VMInteger_new_with(version));   
}

void  _Image_getWordLength(pVMObject object, pVMFrame frame) {
    pVMImage img = (pVMImage)SEND(frame, pop);
    int32_t wordlength = (time_t)SEND(img, get_word_length);
    SEND(frame, push, (pVMObject)VMInteger_new_with(wordlength));   
}

void  _Image_isCompressed(pVMObject object, pVMFrame frame) {
    pVMImage img = (pVMImage)SEND(frame, pop);
    
    bool b = SEND(img, is_compressed);
    pVMObject vmBool = (b) ? true_object : false_object;
    
    SEND(frame, push, vmBool);
}

void  _Image_isEncrypted(pVMObject object, pVMFrame frame) {
    pVMImage img = (pVMImage)SEND(frame, pop);
    
    bool b = SEND(img, is_encrypted);
    pVMObject vmBool = (b) ? true_object : false_object;
    
    SEND(frame, push, vmBool);
}

void  _Image_isLoadable(pVMObject object, pVMFrame frame) {
    pVMImage img = (pVMImage)SEND(frame, pop);
    
    bool b = SEND(img, is_loadable);
    pVMObject vmBool = (b) ? true_object : false_object;
    
    SEND(frame, push, vmBool);
}

}
