/*
 *
Copyright (c) 2007 Michael Haupt, Tobias Pape
Software Architecture Group, Hasso Plattner Institute, Potsdam, Germany
http://www.hpi.uni-potsdam.de/swa/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
  */

#include "VMImage.h"

#include <module.Memory>

//
//  Class Methods (Starting with VMImage_) 
//


/** 
 * Create a new VMImage
 */
pVMImage VMImage_new(void) {  
    ImageHeader image_header;
    ImageInfoHeader info_header;
    
    return VMImage_new_with((pVMInteger)nil_object, (pVMString)nil_object, image_header, info_header);
}


/** 
 * Create a new VMImage with initial value
 * todo: file_name should be a c-level string, otherwise it might get garbage collected
 */
pVMImage VMImage_new_with(const pVMInteger file_size, const pVMString file_name, const ImageHeader image_header,
		const ImageInfoHeader info_header) {
	
    pVMImage result = (pVMImage)memory_allocate_object(sizeof(VMImage));
    if (result) {
        result->_vtable = VMImage_vtable();
        size_t number_of_fields = NUMBER_OF_OBJECT_FIELDS(VMObject) + 2;
        INIT1(result, (void*)&number_of_fields);
        
        result->image_size = file_size;
        result->file_name = file_name;
        result->image_header = image_header;
        result->info_header = info_header;
        result->hash = result->info_header.timestamp_of_storage;
        
        SEND(result, set_class, image_class);
    }
    return result;
}


//
//  Instance Methods (Starting with _VMImage_) 
//

ImageHeader _VMImage_get_image_header(void* _self) {
    pVMImage self = (pVMImage)_self;
    
    return self->image_header;
}

ImageInfoHeader _VMImage_get_info_header(void* _self) {
    pVMImage self = (pVMImage)_self;
    
    return self->info_header;
}

pVMString _VMImage_get_file_name(void* _self) {
    pVMImage self = (pVMImage)_self;
    
    return self->file_name;
}

pVMInteger _VMImage_get_size(void* _self) {
    pVMImage self = (pVMImage)_self;
    
    return self->image_size;
}

time_t _VMImage_get_timestamp(void* _self) {
    pVMImage self = (pVMImage)_self;
    
    return self->info_header.timestamp_of_storage;
}

int32_t _VMImage_get_version(void* _self) {
    pVMImage self = (pVMImage)_self;
    
    return self->image_header.version;
}

int32_t _VMImage_get_word_length(void* _self) {
    pVMImage self = (pVMImage)_self;
    
    return self->image_header.word_length;
}

bool _VMImage_is_compressed(void* _self) {
    pVMImage self = (pVMImage)_self;
    
    return Image_is_compressed(self);
}

bool _VMImage_is_encrypted(void* _self) {
    pVMImage self = (pVMImage)_self;
    
    return Image_is_encrypted(self);
}

bool _VMImage_is_loadable(void* _self) {
    pVMImage self = (pVMImage)_self;
    
    return Image_is_loadable(self);
}
