/* Native multi-threading for CSOM - Signal primitive class
 * 
 * Copyright (C) 2007 by Martin Beck and Jan Klimke
 * 
 * created 2007-06-11
 */
 
#include <module.NativeThreads>
#include <module.Memory>
#include <module.VM>

#include <stdbool.h>
#include <stdio.h>

pSignalId _VMSignal_get_embedded_signal_id(pVMSignal);
pMutexId  _VMSignal_get_embedded_mutex_id(pVMSignal);
void      _VMSignal_wait(pVMSignal);
void      _VMSignal_signal(pVMSignal);
bool      _VMSignal_signal_all(pVMSignal);

void _VMSignal_init(void* _self, ...);
void _VMSignal_free(void *self);

pVMSignal VMSignal_new() {
	pVMSignal result = (pVMSignal) memory_allocate(sizeof(VMSignal));
    if(result) {
        result->_vtable = VMSignal_vtable();
        INIT(result);
    }
    return result;
}

void _VMSignal_init(void* _self, void* param1, void* param2) {
	pVMSignal self = (pVMSignal)_self;
	
	size_t size = NUMBER_OF_OBJECT_FIELDS(VMObject);
	SUPER(VMObject, self, init, &size, NULL);
    
	// create a new pthread signal and embed it in the CSOM signal
	int err = pthread_cond_init(&(self->embedded_signal_id), NULL);
	if(err != 0){
        fprintf(stderr, "Could not initialise signal.\n");
	}
	
	err = pthread_mutex_init(&(self->embedded_mutex_id), NULL);
	
	SEND(self, set_class, signal_class);
}

void _VMSignal_free(void *self) {
    // nothing to declare
}

pSignalId _VMSignal_get_embedded_signal_id(void *self) {
	pVMSignal signal_object = (pVMSignal)self;
	return &signal_object->embedded_signal_id;
}

void _VMSignal_set_embedded_signal_id(void *self, SignalId value) {
	pVMSignal signal_object = (pVMSignal)self;
	signal_object->embedded_signal_id = value;
}

pMutexId _VMSignal_get_embedded_mutex_id(void *self) {
	pVMSignal signal_object = (pVMSignal)self;
	return &signal_object->embedded_mutex_id;
}

void _VMSignal_wait(void *self) {
	pVMSignal signal_object = (pVMSignal)self;
	pSignalId psid = SEND(signal_object, get_embedded_signal_id);
	pMutexId pmid = SEND(signal_object, get_embedded_mutex_id);
	pthread_mutex_lock(pmid);
	pthread_cond_wait(psid, pmid);
	pthread_mutex_unlock(pmid);
}


void _VMSignal_signal(void *self) {
	pVMSignal signal_object = (pVMSignal)self;
	pSignalId psid = SEND(signal_object, get_embedded_signal_id);
	pMutexId pmid = SEND(signal_object, get_embedded_mutex_id);
	pthread_mutex_lock(pmid);
	pthread_cond_signal(psid);
	pthread_mutex_unlock(pmid);
}

bool _VMSignal_signal_all(void* _self) {
	pVMSignal self = (pVMSignal)_self;
	return pthread_cond_broadcast(&self->embedded_signal_id);
}
